/*
 * Copyright (C) 2010 SENJU Jiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#define _GNU_SOURCE
#define _REENTRANT

#include <sys/stat.h>
#include <sys/types.h>
#include <aio.h>
#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "lib/lib.h"

#ifndef _POSIX_ASYNCHRONOUS_IO
#error AIO is unavailable.
#endif

int main(int argc, char *argv[])
{
	int err, sigfd;
	char a[BUFSIZ];
	ssize_t ssz;
	struct aiocb aio = {
		.aio_offset	= 0,
		.aio_buf	= a,
		.aio_nbytes	= sizeof(a),
		.aio_reqprio	= 0,
		.aio_sigevent	= {
			.sigev_notify		= SIGEV_SIGNAL,
			.sigev_signo		= SIGRTMIN + 1
		}
	};
	sigset_t mask;
	struct signalfd_siginfo ssi;

	aio.aio_fildes = open(argv[1], O_RDONLY);
	assert(aio.aio_fildes >= 0);
	aio.aio_sigevent.sigev_value.sival_int = aio.aio_fildes;

	sigemptyset(&mask);
	sigaddset(&mask, aio.aio_sigevent.sigev_signo);
	err = sigprocmask(SIG_BLOCK, &mask, NULL);
	assert(!err);
	sigfd = signalfd(-1, &mask, /*flags unused*/0);
	assert(sigfd >= 0);

	err = aio_read(&aio);
	assert(!err);

	/* do other jobs */

	ssz = read(sigfd, &ssi, sizeof(ssi));
	assert(ssz == sizeof(ssi));
	assert(ssi.ssi_signo == aio.aio_sigevent.sigev_signo);
	assert(ssi.ssi_int == aio.aio_fildes);
	printf("%s: aio_read from fd %d completed\n",
	       __func__, ssi.ssi_int);
	err = aio_error(&aio);
	assert(err != EINPROGRESS);
	if (!err) {
		ssz = aio_return(&aio);
		printf("%s, %zd read\n", argv[1], ssz);
	}

	return 0;
}
