/*
 * Copyright (C) 2014 SENJU Jiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <assert.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "libtp.h"

int main(int argc, char *argv[])
{
	int err, fd, do_close;
	char pat[16], *fname, *method, *p;
	struct lock_ops *ops;
	pid_t pid;

	fname = argv[1];
	method = argv[2];
	ops = lock_ops(method);
	p = argv[3];
	if (!strcmp(p, "unlock"))
		do_close = 0;
	else if (!strcmp(p, "close"))
		do_close = 1;
	else
		assert(0);

	fd = open_and_pat(fname, O_RDWR, pat, sizeof(pat));
	assert(fd >= 0);
	err = ops->lock(fd, NULL);
	assert(!err);
	GREP_LOCK(pat);

	pid = fork();
	assert(pid >= 0);
	if (!pid) {
		if (do_close) {
			printf("pid %d closes\n", getpid());
			err = close(fd);
		} else
			err = ops->unlock(fd, NULL);
		assert(!err);
		exit(0);
	} else
		wait(NULL);
	GREP_LOCK(pat);

	return 0;
}
