/*
 * Copyright (C) 2014 SENJU Jiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <assert.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "libtp.h"

struct lock_ops *ops;

static void lock_it(char *fname)
{
	int err, fd;
	char pat[16];

	fd = open_and_pat(fname, O_RDWR, pat, sizeof(pat));
	assert(fd >= 0);

	err = ops->lock(fd, NULL);
	assert(!err);

	/* do something */
	sleep(1);
	GREP_LOCK(pat);

	err = ops->unlock(fd, NULL);
	assert(!err);
}

int main(int argc, char *argv[])
{
	int i;
	pid_t pid[2];
	char *fname, *method;

	fname = argv[1];
	method = argv[2];
	ops = lock_ops(method);

	for (i = 0; i < 2; i++) {
		pid[i] = fork();
		assert(pid[i] >= 0);
		if (!pid[i]) {
			lock_it(fname);
			exit(0);
		}
	}

	for (i = 0; i < 2; i++)
		wait(NULL);

	return 0;
}
