/*
 * Copyright (C) 2014 SENJU Jiro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <sys/stat.h>
#include <sys/types.h>
#include <assert.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/*
  copied from linux-3.16/include/uapi/asm-generic/fcntl.h.
  the values depend upon the cpu architecture.
*/
#ifndef O_PATH
#define O_PATH		010000000
#endif

#ifndef __O_TMPFILE
#define __O_TMPFILE	020000000
#endif

#ifndef O_TMPFILE
#define O_TMPFILE (__O_TMPFILE | O_DIRECTORY)
#endif

int main(int argc, char *argv[])
{
	int err, tmpfile, len, dirfd;
	pid_t pid;
	struct stat st;
	dev_t dev;
	ino_t ino;
	char proc[16], cmd[32], fdpath[16], msg[] = "hello, world", newfile[64];
	ssize_t ssz;

	tmpfile = open(argv[1], O_RDWR | O_TMPFILE, S_IRUSR | S_IWUSR);
	assert(tmpfile >= 0);
	err = fstat(tmpfile, &st);
	assert(!err);
	assert(!st.st_nlink);	/* the link count is zero */
	dev = st.st_dev;
	ino = st.st_ino;
	ssz = write(tmpfile, msg, sizeof(msg) - 1);
	assert(ssz == sizeof(msg) - 1);

	pid = getpid();
	len = snprintf(proc, sizeof(proc), "/proc/%d/fd", pid);
	assert(len < sizeof(proc));
	len = snprintf(cmd, sizeof(cmd), "ls -l %s", proc);
	assert(len < sizeof(cmd));
	err = system(cmd);
	assert(!err);

	len = snprintf(fdpath, sizeof(fdpath), "%s/%d", proc, tmpfile);
	assert(len < sizeof(cmd));
	err = stat(fdpath, &st);
	assert(!err);
	assert(!st.st_nlink);		/* the link count is zero */
	assert(st.st_dev == dev);	/* should be same to tmpfile */
	assert(st.st_ino == ino);
	assert(st.st_size == ssz);

#if 1
	dirfd = open(argv[1], O_PATH);
	assert(dirfd >= 0);
	err = linkat(AT_FDCWD, fdpath, dirfd, argv[2], AT_SYMLINK_FOLLOW);
	assert(!err);
	len = snprintf(newfile, sizeof(newfile), "%s/%s", argv[1], argv[2]);
	assert(len < sizeof(newfile));
#else
	len = snprintf(newfile, sizeof(newfile), "%s/%s", argv[1], argv[2]);
	assert(len < sizeof(newfile));
	err = linkat(AT_FDCWD, fdpath, AT_FDCWD, newfile, AT_SYMLINK_FOLLOW);
	assert(!err);
#endif

	err = stat(newfile, &st);
	assert(!err);
	assert(st.st_nlink == 1);	/* the link count is zero */
	assert(st.st_dev == dev);	/* should be same to tmpfile */
	assert(st.st_ino == ino);
	assert(st.st_size == ssz);
	err = system(cmd);
	assert(!err);

	return 0;
}
