
#define _GNU_SOURCE
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>
#include <pthread.h>
#include <stdio.h>
#include <unistd.h>

struct th_arg {
	int src, dst;
	off_t size;
};

void *splice_th(void *_arg)
{
	struct th_arg *arg = _arg;
	ssize_t ssz;

	ssz = splice(arg->src, NULL, arg->dst, NULL, arg->size, 0);
	if (ssz < 0)
		perror(NULL);
	return (ssz < 0) ? (void *)-1 : NULL;
}

int copy_file(char *dst, char *src)
{
	int err, pfd[2];
	struct {
		pthread_t id;
		struct th_arg arg;
	} th[2];
	struct stat st;

	err = pipe(pfd);
	if (err) {
		perror("pipe");
		goto out;
	}

	err = -1;
	th[0].arg.dst = pfd[1];
	th[0].arg.src = open(src, O_RDONLY);
	if (th[0].arg.src < 0) {
		perror(src);
		goto out;
	}
	err = fstat(th[0].arg.src, &st);
	if (err) {
		perror(src);
		goto out;
	}

	err = -1;
	th[0].arg.size = st.st_size;
	errno = pthread_create(&th[0].id, NULL, splice_th, &th[0].arg);
	if (errno) {
		perror(NULL);
		goto out;
	}

	th[1].arg.size = st.st_size;
	th[1].arg.src = pfd[0];
	th[1].arg.dst = open(dst, O_WRONLY | O_CREAT | O_EXCL,
			     S_IRUSR | S_IWUSR | S_IRWXG | S_IRWXO);
	if (th[1].arg.dst < 0) {
		perror(dst);
		goto out_th;
	}
	errno = pthread_create(&th[1].id, NULL, splice_th, &th[1].arg);
	if (errno) {
		perror(NULL);
		goto out_th;
	}

	pthread_join(th[1].id, (void *)&err);
	close(pfd[0]);
	close(pfd[1]);

out_th:
	pthread_cancel(th[0].id);
	pthread_join(th[0].id, NULL);
out:
	return err;
}

int main(int argc, char *argv[])
{
	return copy_file(argv[2], argv[1]);
}

/*
 * Local variables: ;
 * compile-command: "gcc -g -Wall -UNDEBUG -pthread fromfd.c";
 * End: ;
 */
