#!/bin/sh

# Copyright (C) 2010 J. R. Okajima

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

Repeat=${Repeat:-3}
set -eu

do_union() # type
{
	echo $1
	seq $Repeat |
	while read n
	do sh ./union.sh $1
	done 2>&1 |
	tee union_$1.result |
	grep '^[0-9]' |
	awk -v type=$1 '
	{
		if (FNR % 2) {
			find["u"] += $1;
			find["s"] += $2;
			find["e"] += $3;
		} else {
			cpup["u"] += $1;
			cpup["s"] += $2;
			cpup["e"] += $3;
		}
	}
	END {
		print type, FNR, "records";
		n = FNR / 2;
		print "find:", find["u"] / n, find["s"] / n, find["e"] / n;
		print "cpup:", cpup["u"] / n, cpup["s"] / n, cpup["e"] / n;
	}'
	sleep 1
}

do_decomp() # type
{
	echo $1
	seq $Repeat |
	while read n
	do sh ./decomp.sh $1
	done 2>&1 |
	tee decomp_$1.result |
	grep '^[0-9]' |
	awk -v type=$1 -v nf=16 -v nl=9 '
	!(FNR % nl) {
		for (i = 1; i <= nf; i++)
			rand_para[i] += $i;
	}
	(FNR % nl) == nl - 1 {
		rand_para_e += $0;
	}
	(FNR % nl) == nl - 2 {
		for (i = 1; i <= nf; i++)
			rand_vm[i] += $i;
	}
	(FNR % nl) == nl - 3 {
		rand_e += $0;
	}
	(FNR % nl) == nl - 4 {
		for (i = 1; i <= nf; i++)
			seq_para[i] += $i;
	}
	(FNR % nl) == nl - 5 {
		seq_para_e += $0;
	}
	(FNR % nl) == nl - 6 {
		for (i = 1; i <= nf; i++)
			seq[i] += $i;
	}
	(FNR % nl) == nl - 7 {
		seq_e += $0;
	}
	(FNR % nl) == nl - 8 {
		fsize = $0;
	}
	END {
		print type, FNR, "records";
		print fsize;

		n = FNR / nl;
		seq_e /= n;
		for (i = 1; i <= nf; i++)
			seq[i] /= n;
		print "seq:", seq_e, seq[13], seq[14], seq[15], seq[16];

		seq_para_e /= n;
		for (i = 1; i <= nf; i++)
			seq_para[i] /= n;
		print "seq_para:", seq_para_e, seq_para[13], seq_para[14], \
			seq_para[15], seq_para[16];

		rand_e /= n;
		for (i = 1; i <= nf; i++)
			rand_vm[i] /= n;
		print "rand:", rand_e, rand_vm[13], rand_vm[14], \
			rand_vm[15], rand_vm[16];

		rand_para_e /= n;
		for (i = 1; i <= nf; i++)
			rand_para[i] /= n;
		print "rand_para:", rand_para_e, rand_para[13], rand_para[14], \
			rand_para[15], rand_para[16];
	}'
}

case $(uname -r) in
*2.6.33*)
	for i in ext2 aufs dm
	do do_union $i
	done

	for i in ext2 sq40 sq40nofrag sq40para sq40nofrag_para cloop sq40nested
	do do_decomp $i
	done
	;;
*2.6.34-rc6*)
	do_decomp sq40lzma
	;;
*2.6.27.4*)
	do_decomp sqlzma34
	;;
esac
